import 'dart:convert';
import 'dart:math';

import "../constants.dart";
import 'package:http/http.dart' as http;

import '../models/dart/gateway.pb.dart';
import '../util/parsers.dart';
// import 'package:flutter/foundation.dart'; // for kReleaseMode

var random = Random.secure();

/// Retrieve a set of gateways provided by default by Vocdoni
Future<BootNodeGateways> getDefaultGatewaysInfo() {
  return getGatewaysInfoFromBootNode(defaultGatewayBootNodesUri);
}

/// Retrieve a set of gateways parsed from the given link
/// The link contents must conform to the schema defined on
/// https://vocdoni.io/docs/#/architecture/components/bootnode
Future<BootNodeGateways> getGatewaysInfoFromBootNode(
    String bootnodesUri) async {
  try {
    var response = await http.get(bootnodesUri.toString());
    if (response == null ||
        response.statusCode < 200 ||
        response.statusCode >= 300)
      throw Exception("The gateway information could not be fetched");

    String strJson = utf8.decode(response.bodyBytes);
    return parseBootnodeGateways(strJson);
  } catch (err) {
    throw Exception("The gateway information could not be fetched");
  }
}

/// Retrieve the parameters of a randomly chosen DVote and Web3 gateway
Future<GatewayInfo> getRandomDefaultGatewayInfo(String networkId) {
  return getRandomGatewayInfo(defaultGatewayBootNodesUri, networkId);
}

Future<GatewayInfo> getRandomGatewayInfo(
    String bootnodesUri, String networkId) async {
  final gws = await getGatewaysInfoFromBootNode(bootnodesUri);

  BootNodeGateways_NetworkNodes nodes =
      networkId == "homestead" ? gws.homestead : gws.goerli;

  if (nodes.dvote.length == 0 && nodes.web3.length == 0) return null;

  GatewayInfo result = GatewayInfo();
  if (nodes.dvote.length > 0) {
    final int dvoteIdx = random.nextInt(nodes.dvote.length);
    result.dvote = nodes.dvote[dvoteIdx].uri;
    result.supportedApis.addAll(nodes.dvote[dvoteIdx].apis);
    result.publicKey = nodes.dvote[dvoteIdx].pubKey;
  }
  if (nodes.web3.length > 0) {
    final int web3Idx = random.nextInt(nodes.web3.length);
    result.web3 = nodes.web3[web3Idx].uri;
  }
  return result;
}
